==> media_entity_slideshow-8.x-2.1.tar.gz: media_entity_slideshow/src/Plugin/media/Source/Slideshow.php <==
<?php

namespace Drupal\media_entity_slideshow\Plugin\media\Source;

use Drupal\Component\Utility\Crypt;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaSourceEntityConstraintsInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Tzsk\Collage\MakeCollage;

/**
 * Provides media type plugin for Slideshows.
 *
 * @MediaSource(
 *   id = "slideshow",
 *   label = @Translation("Slideshow"),
 *   description = @Translation("Provides business logic and metadata for slideshows."),
 *   default_thumbnail_filename = "slideshow.png",
 *   allowed_field_types = {"entity_reference"},
 * )
 */
class Slideshow extends MediaSourceBase implements MediaSourceEntityConstraintsInterface {

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The logger channel for media.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->setFileSystem($container->get('file_system'));
    return $instance;
  }

  /**
   * Set the filesystem service.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The filesystem service.
   */
  protected function setFileSystem(FileSystemInterface $file_system) {
    $this->fileSystem = $file_system;
  }

  /**
   * Set the logger service.
   *
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  protected function setLogger(LoggerInterface $logger) {
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes() {
    $attributes = [
      'length' => $this->t('Slideshow length'),
    ];

    return $attributes;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['thumbnails_directory'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Thumbnails location'),
      '#default_value' => $this->configuration['thumbnails_directory'],
      '#description' => $this->t('Thumbnails will be generated for local usage. This is the URI of the directory where they will be placed.'),
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'thumbnails_directory' => 'public://slideshow_thumbnails',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadata(MediaInterface $media, $name) {
    $source_field = $this->configuration['source_field'];

    switch ($name) {
      case 'default_name':
        // The default name will be the timestamp + number of slides.
        $length = $this->getMetadata($media, 'length');
        if (!empty($length)) {
          return $this->formatPlural($length,
            '1 slide, created on @date',
            '@count slides, created on @date',
            [
              '@date' => \Drupal::service('date.formatter')
                ->format($media->getCreatedTime(), 'custom', DateTimeItemInterface::DATETIME_STORAGE_FORMAT),
            ]);
        }
        return parent::getMetadata($media, 'default_name');

      case 'length':
        return $media->{$source_field}->count();

      case 'thumbnail_uri':
        if ($media->{$source_field}->isEmpty()) {
          return parent::getMetadata($media, 'thumbnail_uri');
        }
        return $this->getLocalThumbnailUri($media);

      default:
        return parent::getMetadata($media, $name);
    }
  }

  /**
   * Returns the local URI for the slideshow thumbnail.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media slidshow item.
   *
   * @return mixed|string|null
   *   The slidshow thumbnail URI.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function getLocalThumbnailUri(MediaInterface $media) {
    $configuration = $this->getConfiguration();
    $source_field = $configuration['source_field'];

    $thumbnails = [];
    for ($i = 0; $i < min(4, $media->{$source_field}->count()); $i++) {
      /** @var \Drupal\media\MediaInterface $slideshow_item */
      $slideshow_item = $this->entityTypeManager->getStorage('media')->load($media->{$source_field}->get($i)->target_id);
      if (!$slideshow_item) {
        return parent::getMetadata($media, 'thumbnail_uri');
      }

      /** @var \Drupal\media\MediaTypeInterface $bundle */
      $bundle = $this->entityTypeManager->getStorage('media_type')->load($slideshow_item->bundle());
      if (!$bundle) {
        return parent::getMetadata($media, 'thumbnail_uri');
      }

      $thumbnails[] = $bundle->getSource()->getMetadata($slideshow_item, 'thumbnail_uri');
    }

    if (!$thumbnails) {
      return parent::getMetadata($media, 'thumbnail_uri');
    }

    // Compute the local thumbnail URI, regardless of whether or not it exists.
    $directory = $configuration['thumbnails_directory'];
    $local_thumbnail_uri = "$directory/" . Crypt::hashBase64(serialize($thumbnails)) . '.jpg';

    // If the local thumbnail already exists, return its URI.
    if (file_exists($local_thumbnail_uri)) {
      return $local_thumbnail_uri;
    }

    // First ensure that the destination directory is writable, and if it's not,
    // log an error and bail out.
    if (!$this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS)) {
      $this->logger->warning('Could not prepare thumbnail destination directory @dir for oEmbed media.', [
        '@dir' => $directory,
      ]);
      return NULL;
    }

    $collage = new MakeCollage();
    /** @var Intervention\Image $image */
    $image = $collage->make(1024, 1024)
      ->padding(10)
      ->from($thumbnails);
    $image->save($local_thumbnail_uri);

    return $local_thumbnail_uri;
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityConstraints() {
    $source_field = $this->configuration['source_field'];

    return ['ItemsCount' => ['sourceFieldName' => $source_field]];
  }

}

==> media_entity_slideshow-8.x-2.3.tar.gz: media_entity_slideshow/src/Plugin/media/Source/Slideshow.php <==
<?php

namespace Drupal\media_entity_slideshow\Plugin\media\Source;

use Drupal\Component\Utility\Crypt;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaSourceEntityConstraintsInterface;
use Drupal\media_entity_slideshow\ThumbnailGenerator;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides media type plugin for Slideshows.
 *
 * @MediaSource(
 *   id = "slideshow",
 *   label = @Translation("Slideshow"),
 *   description = @Translation("Provides business logic and metadata for slideshows."),
 *   default_thumbnail_filename = "slideshow.png",
 *   allowed_field_types = {"entity_reference"},
 * )
 */
class Slideshow extends MediaSourceBase implements MediaSourceEntityConstraintsInterface {

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The logger channel for media.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The thumbnail generator.
   *
   * @var \Drupal\media_entity_slideshow\ThumbnailGenerator
   */
  protected $thumbnailGenerator;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->setFileSystem($container->get('file_system'));
    $instance->setLogger($container->get('logger.factory'));
    $instance->setThumbnailGenerator($container->get('media_entity_slideshow.thumbnail_generator'));
    return $instance;
  }

  /**
   * Set the filesystem service.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The filesystem service.
   */
  protected function setFileSystem(FileSystemInterface $file_system) {
    $this->fileSystem = $file_system;
  }

  /**
   * Set the logger service.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   */
  protected function setLogger(LoggerChannelFactoryInterface $logger_factory) {
    $this->logger = $logger_factory->get('media_entity_slideshow');
  }

  /**
   * Set the thumbnail generator service.
   *
   * @param \Drupal\media_entity_slideshow\ThumbnailGenerator $thumbnail_generator
   *   The thumbnail generator.
   */
  protected function setThumbnailGenerator(ThumbnailGenerator $thumbnail_generator) {
    $this->thumbnailGenerator = $thumbnail_generator;
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes() {
    $attributes = [
      'length' => $this->t('Slideshow length'),
    ];

    return $attributes;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['thumbnails_directory'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Thumbnails location'),
      '#default_value' => $this->configuration['thumbnails_directory'],
      '#description' => $this->t('Thumbnails will be generated for local usage. This is the URI of the directory where they will be placed.'),
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'thumbnails_directory' => 'public://slideshow_thumbnails',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadata(MediaInterface $media, $name) {
    $source_field = $this->configuration['source_field'];

    switch ($name) {
      case 'default_name':
        // The default name will be the timestamp + number of slides.
        $length = $this->getMetadata($media, 'length');
        if (!empty($length)) {
          return $this->formatPlural($length,
            '1 slide, created on @date',
            '@count slides, created on @date',
            [
              '@date' => \Drupal::service('date.formatter')
                ->format($media->getCreatedTime(), 'custom', DateTimeItemInterface::DATETIME_STORAGE_FORMAT),
            ]);
        }
        return parent::getMetadata($media, 'default_name');

      case 'length':
        return $media->{$source_field}->count();

      case 'thumbnail_uri':
        if ($media->{$source_field}->isEmpty()) {
          return parent::getMetadata($media, 'thumbnail_uri');
        }
        return $this->getLocalThumbnailUri($media);

      default:
        return parent::getMetadata($media, $name);
    }
  }

  /**
   * Returns the local URI for the slideshow thumbnail.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media slidshow item.
   *
   * @return mixed|string|null
   *   The slidshow thumbnail URI.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function getLocalThumbnailUri(MediaInterface $media) {
    $configuration = $this->getConfiguration();
    $source_field = $configuration['source_field'];

    $thumbnails = [];
    for ($i = 0; $i < min(4, $media->{$source_field}->count()); $i++) {
      /** @var \Drupal\media\MediaInterface $slideshow_item */
      $slideshow_item = $this->entityTypeManager->getStorage('media')->load($media->{$source_field}->get($i)->target_id);
      if (!$slideshow_item) {
        return parent::getMetadata($media, 'thumbnail_uri');
      }

      /** @var \Drupal\media\MediaTypeInterface $bundle */
      $bundle = $this->entityTypeManager->getStorage('media_type')->load($slideshow_item->bundle());
      if (!$bundle) {
        return parent::getMetadata($media, 'thumbnail_uri');
      }

      $thumbnails[] = $bundle->getSource()->getMetadata($slideshow_item, 'thumbnail_uri');
    }

    if (!$thumbnails) {
      return parent::getMetadata($media, 'thumbnail_uri');
    }

    // Compute the local thumbnail URI, regardless of whether or not it exists.
    $directory = $configuration['thumbnails_directory'];
    $local_thumbnail_uri = "$directory/" . Crypt::hashBase64(serialize($thumbnails)) . '.jpg';

    // If the local thumbnail already exists, return its URI.
    if (file_exists($local_thumbnail_uri)) {
      return $local_thumbnail_uri;
    }

    // First ensure that the destination directory is writable, and if it's not,
    // log an error and bail out.
    if (!$this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS)) {
      $this->logger->warning('Could not prepare thumbnail destination directory @dir for oEmbed media.', [
        '@dir' => $directory,
      ]);
      return NULL;
    }

    $this->thumbnailGenerator
      ->setWidth(1024)
      ->setHeight(1024)
      ->setPadding(10)
      ->create($thumbnails)
      ->save($local_thumbnail_uri);

    return $local_thumbnail_uri;
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityConstraints() {
    $source_field = $this->configuration['source_field'];

    return ['ItemsCount' => ['sourceFieldName' => $source_field]];
  }

}
